@@ -19,18 +19,24 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "audio_types.h"
+
+typedef struct {
+  int nr_ch;
+  ChannelType_t *chtype;
+} channel_config_t;
+
 int parse_config(void);
 
 char *get_audio_device(void);
 char *get_audio_driver(void);
 char *get_audio_alsa_name(void);
 
-int get_num_sub_speakers(void);
-int get_num_rear_speakers(void);
-int get_num_front_speakers(void);
+int get_channel_configs(channel_config_t **conf);
 
 double get_a52_level(void);
 int get_a52_drc(void);
+int get_a52_stereo_mode(void);
 
 char *get_sync_type(void);
 int get_sync_resample(void);
