@@ -5,6 +5,17 @@
 
 /* Ugly hack to get the right identifier in the messages */
 extern char *program_name;
+extern int dlevel;
+#define DEFAULT_DLEVEL 3
+
+#define GET_DLEVEL() \
+  {\
+    char *dlevel_str = getenv("OGLE_DLEVEL"); \
+    if(dlevel_str != NULL) \
+      dlevel = atoi(dlevel_str); \
+    else \
+      dlevel = DEFAULT_DLEVEL; \
+  }
 
 #ifdef DEBUG
 
@@ -25,34 +36,58 @@ int debug_indent_level;
 #if defined(__GNUC__) && ( __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 95))
 
 #define FATAL(str, args...) \
-fprintf(stderr, "FATAL[%s]: " str, program_name , ## args)
+do { \
+if(dlevel > 0) fprintf(stderr, "FATAL[%s]: " str, program_name , ## args); \
+} while(0)
+
+#define FATALC(str, args...) \
+do { \
+if(dlevel > 0) fprintf(stderr, str, ## args); \
+} while(0)
 
 #define ERROR(str, args...) \
-fprintf(stderr, "ERROR[%s]: " str, program_name , ## args)
-
+do { \
+if(dlevel > 1) fprintf(stderr, "ERROR[%s]: " str, program_name , ## args); \
+} while(0)
+ 
 #define WARNING(str, args...) \
-fprintf(stderr, "WARNING[%s]: " str, program_name , ## args)
-
+do { \
+if(dlevel > 2) fprintf(stderr, "WARNING[%s]: " str, program_name , ## args); \
+} while(0)
+ 
 #define NOTE(str, args...) \
-fprintf(stderr, "Note[%s]: " str, program_name , ## args)
+do { \
+if(dlevel > 3) fprintf(stderr, "Note[%s]: " str, program_name , ## args); \
+} while(0)
 
 #define DNOTE(str, args...) \
-fprintf(stderr, "Debug[%s]: " str, program_name , ## args)
+do { \
+if(dlevel > 4) fprintf(stderr, "Debug[%s]: " str, program_name , ## args); \
+} while(0)
+
+#define DNOTEC(str, args...) \
+do { \
+if(dlevel > 4) fprintf(stderr, str, ## args); \
+} while(0)
 
 #ifdef DEBUG
 
 #define DPRINTFI(level, text...) \
+do { \
 if(debug >= level) \
 { \
   fprintf(stderr, "%*s", debug_indent_level, ""); \
   fprintf(stderr, ## text); \
-}
+} \
+} while(0) 
 
 #define DPRINTF(level, text...) \
+do { \
 if(debug >= level) \
 { \
   fprintf(stderr, ## text); \
-}
+} \
+} while(0)
 
 #else /* DEBUG */
 
@@ -67,36 +102,60 @@ if(debug >= level) \
 /* TODO: these defines are not portable */
 
 #define FATAL(format, ...) \
-fprintf(stderr, "FATAL[%s]: " format, program_name , ##__VA_ARGS__)
+do { \
+if(dlevel > 0) fprintf(stderr, "FATAL[%s]: " format, program_name , ##__VA_ARGS__); \
+} while(0)
+
+#define FATALC(format, ...) \
+do { \
+if(dlevel > 0) fprintf(stderr, format, ##__VA_ARGS__); \
+} while(0)
+
 
 #define ERROR(format, ...) \
-fprintf(stderr, "ERROR[%s]: " format, program_name , ##__VA_ARGS__)
+do { \
+if(dlevel > 1) fprintf(stderr, "ERROR[%s]: " format, program_name , ##__VA_ARGS__); \
+} while(0)
 
 #define WARNING(format, ...) \
-fprintf(stderr, "WARNING[%s]: " format, program_name , ##__VA_ARGS__)
+do { \
+if(dlevel > 2) fprintf(stderr, "WARNING[%s]: " format, program_name , ##__VA_ARGS__); \
+} while(0)
 
 #define NOTE(format, ...) \
-fprintf(stderr, "Note[%s]: " format, program_name , ##__VA_ARGS__)
+do { \
+if(dlevel > 3) fprintf(stderr, "Note[%s]: " format, program_name , ##__VA_ARGS__); \
+} while(0)
 
 #define DNOTE(format, ...) \
-fprintf(stderr, "Debug[%s]: " format, program_name , ##__VA_ARGS__)
+do { \
+if(dlevel > 4) fprintf(stderr, "Debug[%s]: " format, program_name , ##__VA_ARGS__); \
+} while(0)
 
+#define DNOTEC(format, ...) \
+do { \
+if(dlevel > 4) fprintf(stderr, format, ##__VA_ARGS__); \
+} while(0)
 
 
 #ifdef DEBUG
 
 #define DPRINTFI(level, ...) \
+do { \
 if(debug >= level) \
 { \
   fprintf(stderr, "%*s", debug_indent_level, ""); \
   fprintf(stderr, __VA_ARGS__); \
-}
+} \
+} while(0)
 
 #define DPRINTF(level, ...) \
+do { \
 if(debug >= level) \
 { \
   fprintf(stderr, __VA_ARGS__); \
-}
+} \
+} while(0)
 
 #else /* DEBUG */
 
@@ -111,12 +170,12 @@ if(debug >= level) \
 #ifdef DEBUG
 
 #define DPRINTBITS(level, bits, value) \
-{ \
+do { \
   int n; \
   for(n = 0; n < bits; n++) { \
     DPRINTF(level, "%u", (value>>(bits-n-1)) & 0x1); \
   } \
-}
+} while(0)
 
 #else /* DEBUG */
 
