@@ -24,229 +24,180 @@
 void
 mlib_Init(void);
 
-void
-mlib_VideoAddBlock_U8_S16 (uint8_t *curr_block,
-			   int16_t *mc_block, 
-                           int32_t stride);
-void
-mlib_VideoCopyRefAve_U8_U8 (uint8_t *curr_block,
-			    uint8_t *ref_block,
-                            int width, int height,
-			    int32_t stride);
 void
 mlib_VideoCopyRef_U8_U8 (uint8_t *curr_block,
-                         uint8_t *ref_block,
-                         int32_t width,
-                         int32_t height,
+                         const uint8_t *ref_block,
+                         int32_t width, int32_t height,
                          int32_t stride);
 void
-mlib_VideoInterpAveX_U8_U8(uint8_t *curr_block, 
-                           uint8_t *ref_block,
-                           int width, int height,
-                           int32_t frame_stride,   
-                           int32_t field_stride);
-void
-mlib_VideoInterpAveY_U8_U8(uint8_t *curr_block, 
-                           uint8_t *ref_block,
-                           int width, int height,
-                           int32_t frame_stride,   
-                           int32_t field_stride);
-void
-mlib_VideoInterpAveXY_U8_U8(uint8_t *curr_block, 
-                            uint8_t *ref_block, 
-                            int width, int height,
-                            int32_t frame_stride,   
-                            int32_t field_stride);
-void
-mlib_VideoInterpX_U8_U8(uint8_t *curr_block, 
-			uint8_t *ref_block,
-			int width, int height,
-			int32_t frame_stride,   
-			int32_t field_stride);
-void
-mlib_VideoInterpY_U8_U8(uint8_t *curr_block, 
-			uint8_t *ref_block,
-			int width, int height,
-			int32_t frame_stride,   
-			int32_t field_stride);
-void
-mlib_VideoInterpXY_U8_U8(uint8_t *curr_block, 
-			 uint8_t *ref_block, 
-			 int width, int height,
-			 int32_t frame_stride,   
-			 int32_t field_stride);
-void
 mlib_VideoCopyRefAve_U8_U8_16x16(uint8_t *curr_block,
-                                 uint8_t *ref_block,
+                                 const uint8_t *ref_block,
                                  int32_t stride);
 void 
 mlib_VideoCopyRefAve_U8_U8_16x8(uint8_t *curr_block,
-				uint8_t *ref_block,
+				const uint8_t *ref_block,
 				int32_t stride);
 void 
 mlib_VideoCopyRefAve_U8_U8_8x8(uint8_t *curr_block,
-			       uint8_t *ref_block,
+			       const uint8_t *ref_block,
 			       int32_t stride);
 void 
 mlib_VideoCopyRefAve_U8_U8_8x4(uint8_t *curr_block,
-			       uint8_t *ref_block,
+			       const uint8_t *ref_block,
 			       int32_t stride);
 void
 mlib_VideoCopyRef_U8_U8_16x16(uint8_t *curr_block,
-			      uint8_t *ref_block,
+			      const uint8_t *ref_block,
 			      int32_t stride);
 void 
 mlib_VideoCopyRef_U8_U8_16x16_multiple(uint8_t *curr_block,
-				       uint8_t *ref_block,
+				       const uint8_t *ref_block,
 				       int32_t stride,
 				       int32_t count);
 void 
 mlib_VideoCopyRef_U8_U8_16x8(uint8_t *curr_block,
-			     uint8_t *ref_block,
+			     const uint8_t *ref_block,
 			     int32_t stride);
 void
 mlib_VideoCopyRef_U8_U8_8x8(uint8_t *curr_block,
-			    uint8_t *ref_block,
+			    const uint8_t *ref_block,
 			    int32_t stride);
 void
 mlib_VideoCopyRef_U8_U8_8x8_multiple(uint8_t *curr_block,
-				     uint8_t *ref_block,
+				     const uint8_t *ref_block,
 				     int32_t stride,
 				     int32_t count);
 void 
 mlib_VideoCopyRef_U8_U8_8x4(uint8_t *curr_block,
-			    uint8_t *ref_block,
+			    const uint8_t *ref_block,
 			    int32_t stride);
 void
 mlib_VideoInterpAveX_U8_U8_16x16(uint8_t *curr_block,
-                                 uint8_t *ref_block,
+                                 const uint8_t *ref_block,
                                  int32_t frame_stride,
                                  int32_t field_stride);
 void 
 mlib_VideoInterpAveX_U8_U8_16x8(uint8_t *curr_block,
-				uint8_t *ref_block,
+				const uint8_t *ref_block,
 				int32_t frame_stride,
 				int32_t field_stride);
 void 
 mlib_VideoInterpAveX_U8_U8_8x8(uint8_t *curr_block,
-			       uint8_t *ref_block,
+			       const uint8_t *ref_block,
 			       int32_t frame_stride,
 			       int32_t field_stride);
 void 
 mlib_VideoInterpAveX_U8_U8_8x4(uint8_t *curr_block,
-			       uint8_t *ref_block,
+			       const uint8_t *ref_block,
 			       int32_t frame_stride,
 			       int32_t field_stride);
 void
 mlib_VideoInterpAveY_U8_U8_16x16(uint8_t *curr_block,
-                                 uint8_t *ref_block,
+                                 const uint8_t *ref_block,
                                  int32_t frame_stride,
                                  int32_t field_stride);
 void 
 mlib_VideoInterpAveY_U8_U8_16x8(uint8_t *curr_block,
-				uint8_t *ref_block,
+				const uint8_t *ref_block,
 				int32_t frame_stride,
 				int32_t field_stride);
 void 
 mlib_VideoInterpAveY_U8_U8_8x8(uint8_t *curr_block,
-			       uint8_t *ref_block,
+			       const uint8_t *ref_block,
 			       int32_t frame_stride,
 			       int32_t field_stride);
 void 
 mlib_VideoInterpAveY_U8_U8_8x4(uint8_t *curr_block,
-			       uint8_t *ref_block,
+			       const uint8_t *ref_block,
 			       int32_t frame_stride,
 			       int32_t field_stride);
 void
 mlib_VideoInterpAveXY_U8_U8_16x16(uint8_t *curr_block,
-				  uint8_t *ref_block,
+				  const uint8_t *ref_block,
 				  int32_t frame_stride,
 				  int32_t field_stride);
 void 
 mlib_VideoInterpAveXY_U8_U8_16x8(uint8_t *curr_block,
-                                 uint8_t *ref_block,
+                                 const uint8_t *ref_block,
                                  int32_t frame_stride,
                                  int32_t field_stride);
 void 
 mlib_VideoInterpAveXY_U8_U8_8x8(uint8_t *curr_block,
-				uint8_t *ref_block,
+				const uint8_t *ref_block,
 				int32_t frame_stride,
 				int32_t field_stride);
 void 
 mlib_VideoInterpAveXY_U8_U8_8x4(uint8_t *curr_block,
-				uint8_t *ref_block,
+				const uint8_t *ref_block,
 				int32_t frame_stride,
 				int32_t field_stride);
 void
 mlib_VideoInterpX_U8_U8_16x16(uint8_t *curr_block,
-			      uint8_t *ref_block,
+			      const uint8_t *ref_block,
 			      int32_t frame_stride,
 			      int32_t field_stride);
 void 
 mlib_VideoInterpX_U8_U8_16x8(uint8_t *curr_block,
-			     uint8_t *ref_block,
+			     const uint8_t *ref_block,
 			     int32_t frame_stride,
 			     int32_t field_stride);
 void 
 mlib_VideoInterpX_U8_U8_8x8(uint8_t *curr_block,
-			    uint8_t *ref_block,
+			    const uint8_t *ref_block,
 			    int32_t frame_stride,
 			    int32_t field_stride);
 void 
 mlib_VideoInterpX_U8_U8_8x4(uint8_t *curr_block,
-			    uint8_t *ref_block,
+			    const uint8_t *ref_block,
 			    int32_t frame_stride,
 			    int32_t field_stride);
 void
 mlib_VideoInterpY_U8_U8_16x16(uint8_t *curr_block,
-			      uint8_t *ref_block,
+			      const uint8_t *ref_block,
 			      int32_t frame_stride,
 			      int32_t field_stride);
 void 
 mlib_VideoInterpY_U8_U8_16x8(uint8_t *curr_block,
-			     uint8_t *ref_block,
+			     const uint8_t *ref_block,
 			     int32_t frame_stride,
 			     int32_t field_stride);
 void 
 mlib_VideoInterpY_U8_U8_8x8(uint8_t *curr_block,
-			    uint8_t *ref_block,
+			    const uint8_t *ref_block,
 			    int32_t frame_stride,
 			    int32_t field_stride);
 void 
 mlib_VideoInterpY_U8_U8_8x4(uint8_t *curr_block,
-			    uint8_t *ref_block,
+			    const uint8_t *ref_block,
 			    int32_t frame_stride,
 			    int32_t field_stride);
 void
 mlib_VideoInterpXY_U8_U8_16x16(uint8_t *curr_block,
-			       uint8_t *ref_block,
+			       const uint8_t *ref_block,
 			       int32_t frame_stride,
 			       int32_t field_stride);
 void 
 mlib_VideoInterpXY_U8_U8_16x8(uint8_t *curr_block,
-			      uint8_t *ref_block,
+			      const uint8_t *ref_block,
 			      int32_t frame_stride,
 			      int32_t field_stride);
 void 
 mlib_VideoInterpXY_U8_U8_8x8(uint8_t *curr_block,
-			     uint8_t *ref_block,
+			     const uint8_t *ref_block,
 			     int32_t frame_stride,
 			     int32_t field_stride);
 void 
 mlib_VideoInterpXY_U8_U8_8x4(uint8_t *curr_block,
-			     uint8_t *ref_block,
+			     const uint8_t *ref_block,
 			     int32_t frame_stride,
 			     int32_t field_stride);
 void
-mlib_VideoIDCT8x8_S16_S16(int16_t *block,
-			  int16_t *coeffs);
-void
 mlib_VideoIDCTAdd_U8_S16(uint8_t *block,
-			 int16_t *coeffs,
+			 const int16_t *coeffs,
 			 int32_t stride);
 void
 mlib_VideoIDCT8x8_U8_S16(uint8_t *block,
-			 int16_t *coeffs,
+			 const int16_t *coeffs,
 			 int32_t stride);
 
 #endif /* C_MLIB_H_INCLUDED */
