@@ -157,7 +157,7 @@ int last_gop_had_repeat_field_progressive_frame = 0;
 
 
 char *program_name;
-
+int dlevel;
 
 
 //prototypes
@@ -169,10 +169,10 @@ void sequence_header(void);
 void sequence_extension(void);
 void sequence_display_extension(void);
 void extension_and_user_data(unsigned int i);
-void picture_coding_extension(void);
+int  picture_coding_extension(void);
 void user_data(void);
 void group_of_pictures_header(void);
-void picture_header(void);
+int  picture_header(void);
 void picture_data(void);
 int  mpeg1_slice(void);
 void mpeg2_slice(void);
@@ -276,7 +276,7 @@ int main(int argc, char **argv)
   int output_client = 0;
   
   program_name = argv[0];
-
+  GET_DLEVEL();
   /* Parse command line options */
   while ((c = getopt(argc, argv, "d:r:f:m:hs")) != EOF) {
     switch (c) {
@@ -322,14 +322,15 @@ int main(int argc, char **argv)
   if(msgqid != -1) {
     MsgEvent_t ev;
     if((msgq = MsgOpen(msgqid)) == NULL) {
-      fprintf(stderr, "video_decoder: couldn't get message q\n");
+      FATAL("%s", "couldn't get message qid\n");
       exit(1);
     }
     
     ev.type = MsgEventQRegister;
     ev.registercaps.capabilities = DECODE_MPEG1_VIDEO | DECODE_MPEG2_VIDEO;
     if(MsgSendEvent(msgq, CLIENT_RESOURCE_MANAGER, &ev, 0) == -1) {
-      fprintf(stderr, "video_decode: register capabilities\n");
+      DNOTE("%s", "register capabilities failed\n");
+      //FIXME should we not retry/fail here?
     }
 
     /*
@@ -339,24 +340,24 @@ int main(int argc, char **argv)
       fprintf(stderr, "video_decode: didn't get video_output\n");
     }
     */
-    fprintf(stderr, "## video_stream:req cap\n");
+    DNOTE("%s", "## req cap\n");
     while(!input_stream) {
-      MsgNextEvent(msgq, &ev);
-      switch(ev.type) {
-      case MsgEventQGntCapability:
-	if(ev.gntcapability.capability & VIDEO_OUTPUT) {
-	  output_client = ev.gntcapability.capclient;
-	} else {
-	  fprintf(stderr, "ui: capabilities not requested\n");
+      if(MsgNextEvent(msgq, &ev) != -1) {
+	switch(ev.type) {
+	case MsgEventQGntCapability:
+	  if(ev.gntcapability.capability & VIDEO_OUTPUT) {
+	    output_client = ev.gntcapability.capclient;
+	  } else {
+	    DNOTE("%s", "got notified about capabilities not requested\n");
+	  }
+	  break;
+	default:
+	  handle_events(msgq, &ev);
+	  break;
 	}
-	break;
-      default:
-	handle_events(msgq, &ev);
-	break;
       }
     }
-
-    fprintf(stderr, "## video_stream:got cap and stream\n");
+    DNOTE("%s", "got cap and stream\n");
 
     
   }
@@ -483,9 +484,15 @@ void video_sequence(void) {
 	  group_of_pictures_header();
 	  extension_and_user_data(1);
 	}
-	picture_header();
+	if(picture_header() == -1) {
+	  next_start_code();
+	  continue;  // Error, we need our picture header.
+	}
 	DINDENT(1);
-	picture_coding_extension();
+	if(picture_coding_extension() == -1) {
+	  next_start_code();
+	  continue;  // Error, we need the coding extension data (or ?) 
+	}
 	
 	extension_and_user_data(2);
 	
@@ -539,7 +546,10 @@ void video_sequence(void) {
 	group_of_pictures_header();
 	extension_and_user_data(1);
       }
-      picture_header();
+      if(picture_header() == -1) {
+	next_start_code();
+	continue;  // Error, we need out picture header
+      }
       extension_and_user_data(1);
       picture_data();
     } while(nextbits(32) == MPEG2_VS_PICTURE_START_CODE ||
@@ -569,7 +579,7 @@ void sequence_header(void)
 
   sequence_header_code = GETBITS(32, "sequence header code");
   if(sequence_header_code != MPEG2_VS_SEQUENCE_HEADER_CODE) {
-    fprintf(stderr, "wrong start_code sequence_header_code: %08x\n",
+    WARNING("wrong start_code sequence_header_code: %08x\n", 
 	    sequence_header_code);
   }
   
@@ -690,7 +700,7 @@ void sequence_header(void)
     break;
   default:
     DPRINTF(2, "Reserved\n");
-    fprintf(stderr, "reserved framerate\n");
+    WARNING("%s", "reserved framerate found in sequence header\n");
     break;
   }
   
@@ -747,7 +757,7 @@ int detach_data_q(int q_shmid, data_q_t **data_q_list)
   }
 
   if(*data_q_p == NULL) {
-    fprintf(stderr, "ERROR[vs]: detach_data_q q_shmid not found\n");
+    ERROR("%s", "detach_data_q q_shmid not found\n");
     return -1;
   }
 
@@ -776,14 +786,14 @@ int detach_data_q(int q_shmid, data_q_t **data_q_list)
   ev.detachq.q_shmid = q_shmid;
 
   if(MsgSendEvent(msgq, CLIENT_RESOURCE_MANAGER, &ev, 0) == -1) {
-    fprintf(stderr, "**video_decoder: couldn't send destroyq\n");
+    ERROR("%s", "couldn't send destroyq\n");
   }
   
   ev.type = MsgEventQDestroyBuf;
   ev.destroybuf.shmid = data_shmid;
 
   if(MsgSendEvent(msgq, CLIENT_RESOURCE_MANAGER, &ev, 0) == -1) {
-    fprintf(stderr, "**video_decoder: couldn't send destroybuf\n");
+    ERROR("%s", "couldn't send destroybuf\n");
   }
 
   return 0;
@@ -839,7 +849,7 @@ int get_output_buffer(data_q_t *data_q,
   yuv_image_t *image_bufs = NULL;
 
 
-  fprintf(stderr,"vs: get ouput buffer\n");
+  DNOTE("%s", "get ouput buffer\n");
   picture_size = ((yuv_size + (pagesize-1))/pagesize*pagesize);
   
   /* Mlib reads ?8? bytes beyond the last pel (in the v-picture), 
@@ -870,22 +880,22 @@ int get_output_buffer(data_q_t *data_q,
   ev.reqbuf.size = buf_size;
   
   if(MsgSendEvent(msgq, CLIENT_RESOURCE_MANAGER, &ev, 0) == -1) {
-    fprintf(stderr, "**video_decoder: couldn't send buffer request\n");
+    ERROR("%s", "couldn't send buffer request\n");
   }
   
   while(1) {
-    MsgNextEvent(msgq, &ev);
-    if(ev.type == MsgEventQGntBuf) {
-      DPRINTF(1, "video_decoder: got buffer id %d, size %d\n",
-	      ev.gntbuf.shmid,
-	      ev.gntbuf.size);
-      data_shmid = ev.gntbuf.shmid;
-      break;
-    } else {
-      handle_events(msgq, &ev);
+    if(MsgNextEvent(msgq, &ev) != -1) {
+      if(ev.type == MsgEventQGntBuf) {
+	DPRINTF(1, "video_decoder: got buffer id %d, size %d\n",
+		ev.gntbuf.shmid,
+		ev.gntbuf.size);
+	data_shmid = ev.gntbuf.shmid;
+	break;
+      } else {
+	handle_events(msgq, &ev);
+      }
     }
   }
-  
 
   if(data_shmid >= 0) {
     if((data_shmaddr = shmat(data_shmid, NULL, SHM_SHARE_MMU)) == (void *)-1) {
@@ -950,21 +960,22 @@ int get_output_buffer(data_q_t *data_q,
     ev.reqpicbuf.data_buf_shmid = data_shmid;
       
     if(MsgSendEvent(msgq, CLIENT_RESOURCE_MANAGER, &ev, 0) == -1) {
-      fprintf(stderr, "**video_decode: couldn't send picbuf request\n");
+      ERROR("%s", "couldn't send picbuf request\n");
     }
     
     /* wait for answer */
     
     while(1) {
-      MsgNextEvent(msgq, &ev);
-      if(ev.type == MsgEventQGntPicBuf) {
-	q_shmid = ev.gntpicbuf.q_shmid;
-	break;
-      } else {
-	handle_events(msgq, &ev);
+      if(MsgNextEvent(msgq, &ev) != -1) {
+	if(ev.type == MsgEventQGntPicBuf) {
+	  q_shmid = ev.gntpicbuf.q_shmid;
+	  break;
+	} else {
+	  handle_events(msgq, &ev);
+	}
       }
     }
-    
+
     if(q_shmid >= 0) {
       if((q_shmaddr = shmat(q_shmid, NULL, SHM_SHARE_MMU)) == (void *)-1) {
 	perror("**video_decode: attach_picq_buffer(), shmat()");
@@ -975,11 +986,11 @@ int get_output_buffer(data_q_t *data_q,
       q_elems = (q_elem_t *)(q_shmaddr+sizeof(q_head_t));
       
     } else {
-      fprintf(stderr, "** video_decode: couldn't get qbuffer\n");
+      ERROR("%s", "couldn't get qbuffer\n");
     }
     
   } else {
-    fprintf(stderr, "** video_decode: couldn't get buffer\n");
+    ERROR("%s", "couldn't get buffer\n");
   }
   
 
@@ -992,48 +1003,48 @@ int get_output_buffer(data_q_t *data_q,
   data_q->image_bufs = image_bufs;
     
 #ifdef HAVE_MMX
-    emms();
+  emms();
 #endif
   
   /* this should not be here */
-  fprintf(stderr, "horizontal_size: %d, vertical_size: %d\n",
-	  seq.horizontal_size, seq.vertical_size);
-  fprintf(stderr, "padded_width: %d, padded_height: %d\n",
-	  padded_width, padded_height);
+  DNOTE("horizontal_size: %d, vertical_size: %d\n",
+	seq.horizontal_size, seq.vertical_size);
+  DNOTE("padded_width: %d, padded_height: %d\n",
+	padded_width, padded_height);
   
-  fprintf(stderr, "frame rate: ");
+  DNOTE("%s", "frame rate: ");
   switch(seq.header.frame_rate_code) {
   case 0x0:
     break;
   case 0x1:
-    fprintf(stderr, "24000/1001 (23.976)\n");
+    DNOTEC("%s", "24000/1001 (23.976)\n");
     break;
   case 0x2:
-    fprintf(stderr, "24\n");
+    DNOTEC("%s", "24\n");
     break;
   case 0x3:
-    fprintf(stderr, "25\n");
+    DNOTEC("%s", "25\n");
     break;
   case 0x4:
-    fprintf(stderr, "30000/1001 (29.97)\n");
+    DNOTEC("%s", "30000/1001 (29.97)\n");
     break;
   case 0x5:
-    fprintf(stderr, "30\n");
+    DNOTEC("%s", "30\n");
     break;
   case 0x6:
-    fprintf(stderr, "50\n");
+    DNOTEC("%s", "50\n");
     break;
   case 0x7:
-    fprintf(stderr, "60000/1001 (59.94)\n");
+    DNOTEC("%s", "60000/1001 (59.94)\n");
     break;
   case 0x8:
-    fprintf(stderr, "60\n");
+    DNOTEC("%s", "60\n");
     break;
   default:
-    fprintf(stderr, "%f (computed)\n",
-	    (double)(seq.header.frame_rate_code)*
-	    ((double)(seq.ext.frame_rate_extension_n)+1.0)/
-	    ((double)(seq.ext.frame_rate_extension_d)+1.0));
+    DNOTEC("%f (computed)\n",
+	   (double)(seq.header.frame_rate_code)*
+	   ((double)(seq.ext.frame_rate_extension_n)+1.0)/
+	   ((double)(seq.ext.frame_rate_extension_d)+1.0));
     break;
   }
 
@@ -1048,7 +1059,7 @@ void sequence_extension(void) {
   
   extension_start_code = GETBITS(32, "extension_start_code");
   if(extension_start_code != MPEG2_VS_EXTENSION_START_CODE) {
-    fprintf(stderr, "wrong start_code extension_start_code: %08x\n",
+    WARNING("wrong start_code extension_start_code: %08x\n",
 	    extension_start_code);
   }
 
@@ -1074,6 +1085,12 @@ void sequence_extension(void) {
   seq.horizontal_size |= (seq.ext.horizontal_size_extension << 12);
   seq.vertical_size |= (seq.ext.vertical_size_extension << 12);
 
+  seq.dpy_ext.display_horizontal_size = 0; //reset
+  seq.dpy_ext.display_vertical_size = 0;
+  
+  //frame_centre_horizontal_offset = 0; // reset
+  //frame_centre_vertical_offset = 0;
+
 #ifdef DEBUG
 	   
   DPRINTFI(2, "profile_and_level_indication: ");
@@ -1223,7 +1240,7 @@ void sequence_extension(void) {
 
 
 /* 6.2.2.2 Extension and user data */
-void extension_and_user_data(unsigned int i) {
+void extension_and_user_data(const unsigned int i) {
   
   DPRINTFI(1, "extension_and_user_data(%u)\n", i);
   DINDENT(2);
@@ -1243,14 +1260,20 @@ void extension_and_user_data(unsigned int i) {
 
 
 /* 6.2.3.1 Picture coding extension */
-void picture_coding_extension(void)
+int picture_coding_extension(void)
 {
   uint32_t extension_start_code;
-  int m,n;
+  unsigned int m,n;
 
   DPRINTFI(1, "picture_coding_extension()\n");
   DINDENT(2);
   extension_start_code = GETBITS(32, "extension_start_code");
+  if(extension_start_code != MPEG2_VS_EXTENSION_START_CODE) {
+    WARNING("wrong start_code extension_start_code: %08x\n",
+	    extension_start_code);
+    DINDENT(-2);
+    return -1;
+  }
   pic.coding_ext.extension_start_code_identifier 
     = GETBITS(4, "extension_start_code_identifier");
 
@@ -1264,7 +1287,7 @@ void picture_coding_extension(void)
     for(m = 0; m < 2; m++) {
       if(pic.coding_ext.f_code[n][m] == 0) {
 	pic.coding_ext.f_code[n][m] = 1;
-	fprintf(stderr, "f_code[%d][%d] == ZERO\n", n, m);
+	WARNING("f_code[%d][%d] == ZERO\n", n, m);
       }
     }
   }
@@ -1359,6 +1382,7 @@ void picture_coding_extension(void)
   }
   DINDENT(-2);
   next_start_code();
+  return 0;
 }
 
 
@@ -1370,6 +1394,7 @@ void user_data(void)
   DPRINTFI(1, "user_data()\n");
   DINDENT(2);
   user_data_start_code = GETBITS(32, "user_data_start_code");
+  /* It's safe, all callers have already tested for the correct start code */
   while(nextbits(24) != 0x000001) {
     GETBITS(8, "user_data");
   }
@@ -1391,7 +1416,7 @@ void group_of_pictures_header(void)
   group_start_code = GETBITS(32, "group_start_code");
 
   if(group_start_code != MPEG2_VS_GROUP_START_CODE) {
-    fprintf(stderr, "wrong start_code group_start_code: %08x\n",
+    WARNING("wrong start_code group_start_code: %08x\n",
 	    group_start_code);
   }
   /* TODO hack for progessive 24 */
@@ -1419,7 +1444,7 @@ void group_of_pictures_header(void)
 
 
 /* 6.2.3 Picture header */
-void picture_header(void)
+int picture_header(void)
 {
   uint32_t picture_start_code;
 
@@ -1430,8 +1455,10 @@ void picture_header(void)
   picture_start_code = GETBITS(32, "picture_start_code");
   
   if(picture_start_code != MPEG2_VS_PICTURE_START_CODE) {
-    fprintf(stderr, "wrong start_code picture_start_code: %08x\n",
+    WARNING("wrong start_code picture_start_code: %08x\n",
 	    picture_start_code);
+    DINDENT(-2);
+    return -1;
   }
   
   /*
@@ -1530,7 +1557,7 @@ void picture_header(void)
     pic.header.forward_f_code = GETBITS(3, "forward_f_code");
     if(pic.header.forward_f_code == 0) {
       pic.header.forward_f_code = 1;
-      fprintf(stderr, " ** forward_f_code == ZERO\n");
+      WARNING("%s", " ** forward_f_code == ZERO\n");
     }
     pic.coding_ext.f_code[0][0] = pic.header.forward_f_code;
     pic.coding_ext.f_code[0][1] = pic.header.forward_f_code;
@@ -1541,7 +1568,7 @@ void picture_header(void)
     pic.header.backward_f_code = GETBITS(3, "backward_f_code");
     if(pic.header.backward_f_code == 0) {
       pic.header.backward_f_code = 1;
-      fprintf(stderr, "** backward_f_code == ZERO\n");
+      WARNING("%s", "** backward_f_code == ZERO\n");
     }
     pic.coding_ext.f_code[1][0] = pic.header.backward_f_code;
     pic.coding_ext.f_code[1][1] = pic.header.backward_f_code;
@@ -1555,6 +1582,7 @@ void picture_header(void)
   
   DINDENT(-2);
   next_start_code();
+  return 0;
 }
 
 
@@ -1610,8 +1638,9 @@ int get_picture_buf(data_q_t *data_q)
 	}
       }
       //fprintf(stderr, "video_decode: waiting for notification1\n");
-      MsgNextEvent(msgq, &ev);
-      handle_events(msgq, &ev);
+      if(MsgNextEvent(msgq, &ev) != -1) {
+	handle_events(msgq, &ev);
+      }
       /*
       for(n = 0; n < picture_ctrl_head->nr_of_dataelems; n++) {
 	fprintf(stderr, "vs: [%d] is_ref: @%d, disp: @%d\n", 
@@ -1642,8 +1671,9 @@ void dpy_q_put(int id, data_q_t *data_q)
     
     while(data_q->q_elems[elem].in_use) {
       //fprintf(stderr, "video_decode: waiting for notification2\n");
-      MsgNextEvent(msgq, &ev);
-      handle_events(msgq, &ev);
+      if(MsgNextEvent(msgq, &ev) != -1) {
+	handle_events(msgq, &ev);
+      }
     }
   }
 
@@ -1717,7 +1747,7 @@ void picture_data(void)
       
       if(bwd_ref_temporal_reference != -1) {
 	
-	fprintf(stderr, "** temporal reference skipped\n");
+	WARNING("%s", "** temporal reference skipped\n");
 	
 	/* If we are in a new GOP and there is an old 
 	   undisplayed bwd_ref_temporal_reference, _don't_ use 
@@ -1992,7 +2022,7 @@ void picture_data(void)
       } else {
 	calc_rt = realtime;
 	timesub(&err_time, &calc_rt, &realtime);
-	fprintf(stderr, "*vs: offset not valid\n");
+	DNOTE("%s", "time offset not valid yet\n");
       }
 
       
@@ -2022,7 +2052,7 @@ void picture_data(void)
     int sar_frac_n = 1;
     int sar_frac_d = 1;
     //double sar = 1.0;
-    uint16_t hsize,vsize;
+    uint16_t hsize, vsize;
     
       //wrong on some dvd's ?
     if(seq.dpy_ext.display_horizontal_size) { 
@@ -2076,6 +2106,29 @@ void picture_data(void)
     }
     pinfos[buf_id].picture.sar_frac_n = sar_frac_n;
     pinfos[buf_id].picture.sar_frac_d = sar_frac_d;
+
+    //TODO add frame_center__offset compensation and
+    // handle bigger display__size than _size
+    if(seq.dpy_ext.display_horizontal_size > 0 &&
+       seq.dpy_ext.display_horizontal_size <= seq.horizontal_size) {      
+      pinfos[buf_id].picture.display_start_x = 
+	(seq.horizontal_size - seq.dpy_ext.display_horizontal_size) / 2;
+      pinfos[buf_id].picture.display_width =
+	seq.dpy_ext.display_horizontal_size; 
+    } else {
+      pinfos[buf_id].picture.display_start_x = 0;
+      pinfos[buf_id].picture.display_width = 0; 
+    }
+    if(seq.dpy_ext.display_vertical_size > 0 &&
+       seq.dpy_ext.display_vertical_size <= seq.vertical_size) {      
+      pinfos[buf_id].picture.display_start_y = 
+	(seq.vertical_size - seq.dpy_ext.display_vertical_size) / 2;
+      pinfos[buf_id].picture.display_height =
+	seq.dpy_ext.display_vertical_size; 
+    } else {
+      pinfos[buf_id].picture.display_start_y = 0;
+      pinfos[buf_id].picture.display_height = 0; 
+    }
   }
   
 
@@ -2147,16 +2200,16 @@ void picture_data(void)
 	}
 	
 	if(bwd_ref_buf_id == -1) {
-	  fprintf(stderr, "decode: **B-frame before any reference frame!!!\n");
+	  WARNING("%s", " **B-frame before any reference frame!!!\n");
 	}
 	
 	if(fwd_ref_buf_id == -1) { // Test closed_gop too....
-	  fprintf(stderr, "decode: B-frame before forward ref frame\n");
+	  WARNING("%s", "B-frame before forward ref frame\n");
 	}
 	
       } else {
 	/* TODO: what should happen if the temporal reference is wrong */
-	fprintf(stderr, "** temporal reference for B-picture incorrect\n");
+	WARNING("%s", "** temporal reference for B-picture incorrect\n");
 	
 	temporal_reference_error =
 	  pic.header.temporal_reference - (last_temporal_ref_to_dpy + 1)%1024;
@@ -2198,7 +2251,7 @@ void picture_data(void)
       }
     } else if(bwd_ref_temporal_reference < (last_temporal_ref_to_dpy+1)%1024) {
       
-      fprintf(stderr, "** temporal reference in I or P picture incorrect\n");
+      WARNING("%s", "** temporal reference in I or P picture incorrect\n");
       
     }
   }
@@ -2211,7 +2264,7 @@ void picture_data(void)
 
 
 /* 6.2.2.2.1 Extension data */
-void extension_data(unsigned int i)
+void extension_data(const unsigned int i)
 {
   DPRINTFI(1, "extension_data(%d)", i);
   DINDENT(2);
@@ -2289,7 +2342,7 @@ void quant_matrix_extension(void)
   DINDENT(2);
   
   if(GETBITS(1, "load_intra_quantiser_matrix")) {
-    int n;
+    unsigned int n;
     intra_inverse_quantiser_matrix_changed = 1;
 
     /* 7.3.1 Inverse scan for matrix download */
@@ -2301,7 +2354,7 @@ void quant_matrix_extension(void)
   }
     
   if(GETBITS(1, "load_non_intra_quantiser_matrix")) {
-    int n;
+    unsigned int n;
     non_intra_inverse_quantiser_matrix_changed = 1;
     
     /*  7.3.1 Inverse scan for matrix download */
@@ -2325,7 +2378,7 @@ void picture_display_extension(void)
   uint16_t frame_centre_horizontal_offset;
   uint16_t frame_centre_vertical_offset;
   uint8_t number_of_frame_centre_offsets;
-  int i;
+  unsigned int i;
 
   DPRINTFI(1, "picture_display_extension()\n");
   DINDENT(2);
