@@ -622,33 +622,54 @@ const vlc_table_t table_b9[] = {
 };
 
 
-
-const uint8_t scan[2][8][8] = {
-  /* Figure 7-1. Definition of scan[0][v][u] */
-  {
-    {  0,  1,  5,  6, 14, 15, 27, 28 },
-    {  2,  4,  7, 13, 16, 26, 29, 42 },
-    {  3,  8, 12, 17, 25, 30, 41, 43 },
-    {  9, 11, 18, 24, 31, 40, 44, 53 },
-    { 10, 19, 23, 32, 39, 45, 52, 54 },
-    { 20, 22, 33, 38, 46, 51, 55, 60 },
-    { 21, 34, 37, 47, 50, 56, 59, 61 },
-    { 35, 36, 48, 49, 57, 58, 62, 63 }
-  },
-  /* Figure 7-2. Definition of scan[1][v][u] */
-  {
-    {  0,  4,  6, 20, 22, 36, 38, 52 },
-    {  1,  5,  7, 21, 23, 37, 39, 53 },
-    {  2,  8, 19, 24, 34, 40, 50, 54 },
-    {  3,  9, 18, 25, 35, 41, 51, 55 },
-    { 10, 17, 26, 30, 42, 46, 56, 60 },
-    { 11, 16, 27, 31, 43, 47, 57, 61 },
-    { 12, 15, 28, 32, 44, 48, 58, 62 },
-    { 13, 14, 29, 33, 45, 49, 59, 63 }
+#if defined(HAVE_ALTIVEC)
+/* Transposed version for the AltiVec IDCT */
+const uint8_t inverse_scan[2][64] = {
+  /* Derived from Figure 7-1. Definition of scan[0][v][u] */
+  {  0,  8,  1,  2,  9, 16, 24, 17,
+    10,  3,  4, 11, 18, 25, 32, 40,
+    33, 26, 19, 12,  5,  6, 13, 20,
+    27, 34, 41, 48, 56, 49, 42, 35,
+    28, 21, 14,  7, 15, 22, 29, 36,
+    43, 50, 57, 58, 51, 44, 37, 30,
+    23, 31, 38, 45, 52, 59, 60, 53,  
+    46, 39, 47, 54, 61, 62, 55, 63
+  }, 
+  /* Derived from Figure 7-2. Definition of scan[1][v][u] */
+  {  0,  1,  2,  3,  8,  9, 16, 17,
+    10, 11,  4,  5,  6,  7, 15, 14,
+    13, 12, 19, 18, 24, 25, 32, 33,
+    26, 27, 20, 21, 22, 23, 28, 29,
+    30, 31, 34, 35, 40, 41, 48, 49,
+    42, 43, 36, 37, 38, 39, 44, 45,
+    46, 47, 50, 51, 56, 57, 58, 59,
+    52, 53, 54, 55, 60, 61, 62, 63,
   }
 };
-
-#ifndef HAVE_MMX 
+#elif defined(HAVE_MMX)
+/* Wierd version for the MMX IDCT */
+const uint8_t inverse_scan[2][64] = {
+  /* Derived from Figure 7-1. Definition of scan[0][v][u] */
+  {  0,  4,  8, 16, 12,  1,  5,  9,
+    20, 24, 32, 28, 17, 13,  2,  6,
+    10, 21, 25, 36, 40, 48, 44, 33,
+    29, 18, 14,  3,  7, 11, 22, 26,
+    37, 41, 52, 56, 60, 49, 45, 34,
+    30, 19, 15, 23, 27, 38, 42, 53,
+    57, 61, 50, 46, 35, 31, 39, 43,
+    54, 58, 62, 51, 47, 55, 59, 63,
+  },
+  {  0,  8, 16, 24,  4, 12,  1,  9,
+    20, 28, 32, 40, 48, 56, 60, 52,
+    44, 36, 25, 17,  5, 13,  2, 10,
+    21, 29, 33, 41, 49, 57, 37, 45,
+    53, 61, 18, 26,  6, 14,  3, 11,
+    22, 30, 34, 42, 50, 58, 38, 46,
+    54, 62, 19, 27,  7, 15, 23, 31,
+    35, 43, 51, 59, 39, 47, 55, 63,
+  }
+};
+#else
 const uint8_t inverse_scan[2][64] = {
   /* Derived from Figure 7-1. Definition of scan[0][v][u] */
   {  0,  1,  8, 16,  9,  2,  3, 10,  
@@ -661,8 +682,7 @@ const uint8_t inverse_scan[2][64] = {
     53, 60, 61, 54, 47, 55, 62, 63
   }, 
   /* Derived from Figure 7-2. Definition of scan[1][v][u] */
-  {
-     0,  8, 16, 24,  1,  9,  2, 10,
+  {  0,  8, 16, 24,  1,  9,  2, 10,
     17, 25, 32, 40, 48, 56, 57, 49,
     41, 33, 26, 18,  3, 11,  4, 12,
     19, 27, 34, 42, 50, 58, 35, 43,
@@ -672,31 +692,6 @@ const uint8_t inverse_scan[2][64] = {
     38, 46, 54, 62, 39, 47, 55, 63
   }
 };
-#else
-const uint8_t inverse_scan[2][64] = {
-  /* Derived from Figure 7-1. Definition of scan[0][v][u] */
-  {
-     0,   8,   1,   2,   9,  16,  24,  17,
-    10,   3,   4,  11,  18,  25,  32,  40,
-    33,  26,  19,  12,   5,   6,  13,  20,
-    27,  34,  41,  48,  56,  49,  42,  35,
-    28,  21,  14,   7,  15,  22,  29,  36,
-    43,  50,  57,  58,  51,  44,  37,  30,
-    23,  31,  38,  45,  52,  59,  60,  53,  
-    46,  39,  47,  54,  61,  62,  55,  63
-  }, 
-  /* Derived from Figure 7-2. Definition of scan[1][v][u] */
-  {
-     0,   1,   2,   3,   8,   9,  16,  17,
-    10,  11,   4,   5,   6,   7,  15,  14,
-    13,  12,  19,  18,  24,  25,  32,  33,
-    26,  27,  20,  21,  22,  23,  28,  29,
-    30,  31,  34,  35,  40,  41,  48,  49,
-    42,  43,  36,  37,  38,  39,  44,  45,
-    46,  47,  50,  51,  56,  57,  58,  59,
-    52,  53,  54,  55,  60,  61,  62,  63,
-  }
-};
 #endif
 
 /* Table 7-6. Relation between quantiser_scale and quantiser_scale_code */
@@ -711,6 +706,30 @@ const uint8_t q_scale[2][32] = {
 
 
 /* 6.3.7 Quant matrix extension */
+#if defined(HAVE_ALTIVEC)
+/* Transposed version for the AltiVec IDCTs */
+const int8_t default_intra_inverse_quantiser_matrix[8][8] = {
+  {  8, 16, 19, 22, 22, 26, 26, 27 },
+  { 16, 16, 22, 22, 26, 27, 27, 29 },
+  { 19, 22, 26, 26, 27, 29, 29, 35 },
+  { 22, 24, 27, 27, 29, 32, 34, 38 },
+  { 26, 27, 29, 29, 32, 35, 38, 46 },
+  { 27, 29, 34, 34, 35, 40, 46, 56 },
+  { 29, 34, 34, 37, 40, 48, 56, 69 },
+  { 34, 37, 38, 40, 48, 58, 69, 83 }
+};
+#elif defined(HAVE_MMX)
+const int8_t default_intra_inverse_quantiser_matrix[8][8] = {
+  {  8, 19, 26, 29, 16, 22, 27, 34 },
+  { 16, 22, 27, 34, 16, 24, 29, 37 },
+  { 19, 26, 29, 34, 22, 27, 34, 38 },
+  { 22, 26, 29, 37, 22, 27, 34, 40 },
+  { 22, 27, 32, 40, 26, 29, 35, 48 },
+  { 26, 29, 35, 48, 27, 32, 40, 58 },
+  { 26, 29, 38, 56, 27, 34, 46, 69 },
+  { 27, 35, 46, 69, 29, 38, 56, 83 }
+};
+#else
 const int8_t default_intra_inverse_quantiser_matrix[8][8] = {
   {  8, 16, 19, 22, 26, 27, 29, 34 },
   { 16, 16, 22, 24, 27, 29, 34, 37 },
@@ -721,6 +740,7 @@ const int8_t default_intra_inverse_quantiser_matrix[8][8] = {
   { 26, 27, 29, 34, 38, 46, 56, 69 },
   { 27, 29, 35, 38, 46, 56, 69, 83 }
 };
+#endif
 
 const int8_t default_non_intra_inverse_quantiser_matrix[8][8] = {
   { 16, 16, 16, 16, 16, 16, 16, 16 },
